<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink">

<xsl:import href="xradar://stylesheets/radar/graph/common-scripts.xsl"/>
<xsl:import href="xradar://stylesheets/radar/graph/subsystem-graph.xsl"/>
<xsl:import href="xradar://stylesheets/util/text/text.wrapping.template.xsl"/>

<xsl:template name="dependency-graph">
	<xsl:param name="text-x-axis"/>
	<xsl:param name="text-y-axis"/>
	<xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="subsystems-nodeset"/>

	<xsl:variable   name="color-ok"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 3]/@color"/>
	<xsl:variable   name="color-warn"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 2]/@color"/>
	<xsl:variable   name="color-ko"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 1]/@color"/>

	<xsl:variable name="subsystem-nodeset" select="xalan:nodeset($subsystems-nodeset)"/>


  	<svg viewBox="0 0 {$width} {$height}"
  		xmlns="http://www.w3.org/2000/svg" xmlns:link="http://www.w3.org/1999/xlink">

  		<xsl:call-template name="common-scripts"/>



		<xsl:variable name="number-of-levels">
			<xsl:for-each select="$subsystem-nodeset/subsystems/subsystem">
				<xsl:sort select="level" order="descending" data-type="number"/>
				<xsl:if test="position() = 1">
					<xsl:value-of select="level"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>

		<xsl:for-each select="$subsystem-nodeset/subsystems/subsystem">
			<xsl:sort select="level" order="ascending" data-type="number"/>
			<xsl:variable name="this-level" select="level"/>
            <xsl:call-template name="subsystem-box">
				<xsl:with-param name="width" select="0.6 * $width"/>
				<xsl:with-param name="height" select="$height"/>
				<xsl:with-param name="subsystem-element" select="current()"/>
				<xsl:with-param name="number-of-levels" select="$number-of-levels"/>
				<xsl:with-param name="number-of-same-level-elements" select="count(../subsystem[level = $this-level])"/>
				<xsl:with-param name="same-level-element-number" select="count(preceding-sibling::subsystem[level = $this-level])+1"/>
				<xsl:with-param name="position" select="position()"/>
                <xsl:with-param name="color">
                    <xsl:choose>
                    	<xsl:when test="normalize-space(illegal-dependencies) &gt; 0"><xsl:value-of select="$color-ok"/></xsl:when>
                    	<xsl:otherwise><xsl:value-of select="$color-ko"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:with-param>
			</xsl:call-template>
		</xsl:for-each>

		<xsl:for-each select="$subsystem-nodeset/subsystems/subsystem">
			<xsl:sort select="level" order="ascending" data-type="number"/>
			<xsl:variable name="this-level" select="level"/>
            <g style="visibility:hidden;">
                <xsl:attribute name="id"><xsl:value-of select="concat(level,'_',count(preceding-sibling::subsystem[level = $this-level])+1)"/></xsl:attribute>
                    <xsl:call-template name="subsystem-dependency-info">
                        <xsl:with-param name="width" select="0.6 * $width"/>
                        <xsl:with-param name="height" select="$height"/>
                        <xsl:with-param name="subsystem-element" select="current()"/>
                        <xsl:with-param name="number-of-levels" select="$number-of-levels"/>
                        <xsl:with-param name="number-of-same-level-elements" select="count(../subsystem[level = $this-level])"/>
                        <xsl:with-param name="same-level-element-number" select="count(preceding-sibling::subsystem[level = $this-level])+1"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template>
                    <xsl:call-template name="subsystem-snapshot-info">
                        <xsl:with-param name="start-pos" select="0.6 * $width"/>
                        <xsl:with-param name="width" select="0.4 * $width"/>
                        <xsl:with-param name="height" select="$height"/>
                        <xsl:with-param name="subsystem-element" select="current()"/>
                        <xsl:with-param name="number-of-levels" select="$number-of-levels"/>
                        <xsl:with-param name="number-of-same-level-elements" select="count(../subsystem[level = $this-level])"/>
                        <xsl:with-param name="same-level-element-number" select="count(preceding-sibling::subsystem[level = $this-level])+1"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template>
            </g>
		</xsl:for-each>
	</svg>
</xsl:template>

<xsl:template name="subsystem-dependency-info">
	<xsl:param name="height"/>
	<xsl:param name="width"/>
	<xsl:param name="subsystem-element"/>
	<xsl:param name="number-of-levels"/>
	<xsl:param name="number-of-same-level-elements"/>
	<xsl:param name="same-level-element-number"/>
	<xsl:param name="position"/>
	<xsl:param name="second-max-dependencies"/>

	<xsl:variable name="H" select="$height div ($number-of-levels)"/>
	<xsl:variable name="W" select="$width div ($number-of-same-level-elements)"/>
	<xsl:variable name="id" select="concat($subsystem-element/level,'_',$same-level-element-number)"/>



          <!-- draw dependency lines -->
          <xsl:for-each select="$subsystem-element/subsystem-dependences/subsystem[(node() &gt; 0) and (@position != $position)]">
	          <xsl:variable name="this-level" select="@level"/>
	          <xsl:variable name="this-same-level-element-number" select="count(preceding-sibling::subsystem[@level = $this-level])+1"/>
	          <xsl:variable name="this-number-of-same-level-elements" select="count(../subsystem[@level = $this-level])"/>
	          <xsl:variable name="this-W" select="$width div ($this-number-of-same-level-elements)"/>

	          <xsl:variable name="from-box-center-x" select="$W * ($same-level-element-number - 1) + 0.5 * $W"/>
	          <xsl:variable name="to-box-center-x" select="$this-W * ($this-same-level-element-number - 1) + 0.5 * $this-W"/>
	 		  <xsl:variable name="from-box-center-y" select="$H * ($subsystem-element/level - 1) + 0.5 * $H"/>
	          <xsl:variable name="to-box-center-y" select="$H * ($this-level - 1) + 0.5 * $H"/>

	          <xsl:variable name="x-sign">
	          	<xsl:choose>
	          		<xsl:when test="(to-box-center-x - from-box-center-x) &gt;= 0">1</xsl:when>
	          		<xsl:otherwise>-1</xsl:otherwise>
	          	</xsl:choose>
	          </xsl:variable>
	          <xsl:variable name="y-sign">
	          	<xsl:choose>
	          		<xsl:when test="(to-box-center-y - from-box-center-y) &gt;= 0">1</xsl:when>
	          		<xsl:otherwise>-1</xsl:otherwise>
	          	</xsl:choose>
	          </xsl:variable>
				  <line>
				  		<xsl:attribute name="stroke-width">
			    			<xsl:value-of select="3"/>
				    	</xsl:attribute>
				    	<xsl:attribute name="stroke">
			    			<xsl:choose>
			    				<xsl:when test="@legal = 'false'">Red</xsl:when>
			    				<xsl:otherwise>Green</xsl:otherwise>
			    			</xsl:choose>
				    	</xsl:attribute>
				    	<xsl:attribute name="x1"><xsl:value-of select="$from-box-center-x"/></xsl:attribute>
				    	<xsl:attribute name="x2"><xsl:value-of select="$to-box-center-x"/></xsl:attribute>
				    	<xsl:attribute name="y1"><xsl:value-of select="$from-box-center-y"/></xsl:attribute>
				    	<xsl:attribute name="y2"><xsl:value-of select="$to-box-center-y"/></xsl:attribute>
				  </line>

          </xsl:for-each>

          <!-- draw dependency info on top of lines -->
          <xsl:for-each select="$subsystem-element/subsystem-dependences/subsystem[(node() &gt; 0) and (@position != $position)]">
	          <xsl:variable name="this-level" select="@level"/>
	          <xsl:variable name="this-same-level-element-number" select="count(preceding-sibling::subsystem[@level = $this-level])+1"/>
	          <xsl:variable name="this-number-of-same-level-elements" select="count(../subsystem[@level = $this-level])"/>
	          <xsl:variable name="this-W" select="$width div ($this-number-of-same-level-elements)"/>

	          <xsl:variable name="from-box-center-x" select="$W * ($same-level-element-number - 1) + 0.5 * $W"/>
	          <xsl:variable name="to-box-center-x" select="$this-W * ($this-same-level-element-number - 1) + 0.5 * $this-W"/>
	 		  <xsl:variable name="from-box-center-y" select="$H * ($subsystem-element/level - 1) + 0.5 * $H"/>
	          <xsl:variable name="to-box-center-y" select="$H * ($this-level - 1) + 0.5 * $H"/>

	          <xsl:variable name="x-sign">
	          	<xsl:choose>
	          		<xsl:when test="(to-box-center-x - from-box-center-x) &gt;= 0">1</xsl:when>
	          		<xsl:otherwise>-1</xsl:otherwise>
	          	</xsl:choose>
	          </xsl:variable>
	          <xsl:variable name="y-sign">
	          	<xsl:choose>
	          		<xsl:when test="(to-box-center-y - from-box-center-y) &gt;= 0">1</xsl:when>
	          		<xsl:otherwise>-1</xsl:otherwise>
	          	</xsl:choose>
	          </xsl:variable>

				  <rect style="stroke-width:2;fill:white;">

		          	<xsl:attribute name="stroke">
		    			<xsl:choose>
		    				<xsl:when test="@legal = 'false'">
		    					red
		    				</xsl:when>
		    				<xsl:otherwise>
		    					green
		    				</xsl:otherwise>
		    			</xsl:choose>
			    	</xsl:attribute>
		          	<xsl:attribute name="width">
		    			<xsl:value-of select="$H * 0.3"/>
			    	</xsl:attribute>
			    	<xsl:attribute name="height">
			    		<xsl:value-of select="$H * 0.3"/>
			    	</xsl:attribute>
		          	<xsl:attribute name="x">
			    		<xsl:value-of select="$to-box-center-x - $H * 0.15"/>
			    	</xsl:attribute>
			    	<xsl:attribute name="y">
			    		<xsl:value-of select="$to-box-center-y - $H * 0.15"/>
			    	</xsl:attribute>
		          </rect>

		          <text font-family="Verdana" fill="black" style="text-anchor: middle;">
			    	<xsl:attribute name="font-size">
		    			<xsl:value-of select="ceiling($width * 0.025)"/>
			    	</xsl:attribute>
			    	<xsl:attribute name="x">
		    			<xsl:value-of select="$to-box-center-x"/>
			    	</xsl:attribute>
			    	<xsl:attribute name="y">
				    	<xsl:value-of select="$to-box-center-y"/>
			    	</xsl:attribute>
			    	<xsl:value-of select="node()"/>
				</text>
          </xsl:for-each>
</xsl:template>

<xsl:template name="subsystem-snapshot-info">
	<xsl:param name="start-pos"/>
    <xsl:param name="height"/>
	<xsl:param name="width"/>
	<xsl:param name="subsystem-element"/>
	<xsl:param name="number-of-levels"/>
	<xsl:param name="number-of-same-level-elements"/>
	<xsl:param name="same-level-element-number"/>
	<xsl:param name="position"/>

    <xsl:variable name="font-size-heading" select="ceiling($width * 0.04)"/>
    <xsl:variable name="font-size-text" select="ceiling($width * 0.03)"/>

    <text style="{concat('font-size:',$font-size-heading,';font-family:Verdana;')}" x="{$start-pos}" y="{0.05 * $height}"><xsl:value-of select="concat('Snapshot - ', $subsystem-element/name)"/></text>

    <xsl:call-template name="svgTextSplit">
          <xsl:with-param name="text" select="$subsystem-element/description"/>
          <xsl:with-param name="lineDiff" select="$font-size-text"/>
          <xsl:with-param name="maxCharactersPerLine" select="60"/>
          <xsl:with-param name="style" select="concat('font-size:',$font-size-text,';font-family:Verdana;')"/>
          <xsl:with-param name="x-pos" select="$start-pos"/>
          <xsl:with-param name="y-pos" select="0.08 * $height"/>
    </xsl:call-template>

    <text style="{concat('font-size:',$font-size-heading,';font-family:Verdana;')}" x="{$start-pos}" y="{0.5 * $height}">Core Metrics</text>
    <xsl:for-each select="$subsystem-element/statistics/stat">
        <text style="{concat('font-size:',$font-size-text,';font-family:Verdana;')}" x="{$start-pos}" y="{0.5 * $height + position() * ($font-size-text+2)}"><xsl:value-of select="concat(@name, ' =')"/></text>
        <text style="{concat('font-size:',$font-size-text,';font-family:Verdana;')}" x="{$start-pos + 0.5 * $width}" y="{0.5 * $height + position() * ($font-size-text+2)}"><xsl:value-of select="@value"/></text>
    </xsl:for-each>


</xsl:template>

</xsl:stylesheet>
