<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
  	extension-element-prefixes="redirect">

	<xsl:import href="xradar://stylesheets/radar/graph/simple-histogram-graph.xsl"/>
	<xsl:import href="xradar://stylesheets/radar/graph/dependency-graph.xsl"/>
	<xsl:import href="xradar://stylesheets/radar/graph/subsystem-graph.xsl"/>
	<xsl:import href="xradar://stylesheets/radar/graph/abstractness-instability-graph.xsl"/>
	<xsl:import href="xradar://stylesheets/radar/statics/report/framework/master-report-common-link.xsl"/>

    	<xsl:key name="duplicated-class" match="duplications/duplication/classref" use="@name"/>

	<xsl:param name="radar-config-file"/>
	<xsl:param name="docs-home"/>
	<xsl:param name="log"/>

	<xsl:variable name="radar-config-doc" select="document($radar-config-file)"/>
	<xsl:variable name="graph-width" select="'400'"/>
	<xsl:variable name="graph-height" select="'300'"/>
	<xsl:variable name="path-to-root" select="'../'"/>

	<xsl:template match="radar-report">

		<xsl:choose>
			<xsl:when test="$log = 'true'">
				<xsl:message>
					radar-config:<xsl:value-of select="$radar-config-file"/>
					docs-home:<xsl:value-of select="$docs-home"/>
					input xml root:<xsl:value-of select="name(*)"/>
				</xsl:message>
			</xsl:when>
		</xsl:choose>
		<redirect:write file="{$docs-home}/img/subsystem-dependency-graph.svg">
			<xsl:apply-templates select="packages" mode="subsystem-dependency-graph"/>
		</redirect:write>

        <redirect:write file="{$docs-home}/img/subsystem-quality-graph.svg">
			<xsl:apply-templates select="packages" mode="subsystem-quality-graph"/>
		</redirect:write>

		<redirect:write file="{$docs-home}/img/abstractness-instability-graph.svg">
			<xsl:apply-templates select="packages" mode="abstractness-instability-graph"/>
		</redirect:write>

		<redirect:write file="{$docs-home}/img/subsystem-size-histogram.svg">
			<xsl:apply-templates select="packages" mode="subsystem-size-histogram-graph"/>
		</redirect:write>

		<redirect:write file="{$docs-home}/img/subsystem-complexity-per-method-histogram.svg">
			<xsl:apply-templates select="packages" mode="subsystem-complexity-per-method-histogram-graph"/>
		</redirect:write>

		<redirect:write file="{$docs-home}/img/subsystem-violations-per-ncss-histogram.svg">
			<xsl:apply-templates select="packages" mode="subsystem-violations-per-ncss-histogram-graph"/>
		</redirect:write>

		<redirect:write file="{$docs-home}/img/subsystem-style-errors-per-ncss-histogram.svg">
			<xsl:apply-templates select="packages" mode="subsystem-style-errors-per-ncss-histogram-graph"/>
		</redirect:write>

		<xsl:if test="(sources/source = 'JUnit') and ((sources/source = 'JCoverage') or (sources/source = 'Cobertura'))">
			<redirect:write file="{$docs-home}/img/subsystem-tests-per-ncss-histogram.svg">
				<xsl:apply-templates select="packages" mode="subsystem-test-count-per-ncss-histogram-graph"/>
			</redirect:write>

			<redirect:write file="{$docs-home}/img/subsystem-test-coverage-histogram.svg">
				<xsl:apply-templates select="packages" mode="subsystem-test-coverage-histogram-graph"/>
			</redirect:write>
		</xsl:if>

        	<redirect:write file="{$docs-home}/img/subsystem-total-activity-histogram.svg">
			<xsl:apply-templates select="packages" mode="subsystem-source-control-histogram-graph">
                		<xsl:with-param name="sc-metric" select="'total-activity'"/>
                		<xsl:with-param name="y-axis-name" select="'Total Activity'"/>
			</xsl:apply-templates>
		</redirect:write>

        	<redirect:write file="{$docs-home}/img/subsystem-error-percentage-histogram.svg">
            		<xsl:apply-templates select="packages" mode="subsystem-source-control-histogram-graph">
                		<xsl:with-param name="sc-metric" select="'error-percentage'"/>
                		<xsl:with-param name="y-axis-name" select="'Error % of Total Activity'"/>
            		</xsl:apply-templates>
		</redirect:write>

        	<redirect:write file="{$docs-home}/img/subsystem-error-classes-histogram.svg">
            		<xsl:apply-templates select="packages" mode="subsystem-source-control-histogram-graph">
                		<xsl:with-param name="sc-metric" select="'error-classes'"/>
                		<xsl:with-param name="y-axis-name" select="'Classes with Error Issues'"/>
            		</xsl:apply-templates>
		</redirect:write>

        	<redirect:write file="{$docs-home}/img/subsystem-change-classes-histogram.svg">
            		<xsl:apply-templates select="packages" mode="subsystem-source-control-histogram-graph">
                		<xsl:with-param name="sc-metric" select="'change-classes'"/>
                		<xsl:with-param name="y-axis-name" select="'Classes with Change Issues'"/>
             		</xsl:apply-templates>
		</redirect:write>


	</xsl:template>


	<xsl:template match="packages" mode="subsystem-test-count-per-ncss-histogram-graph">
		<xsl:variable name="packages-element" select="current()"/>

		<xsl:call-template name="simple-histogram-graph">
	  		<xsl:with-param name="text-x-axis" 		select="'Subsystem'"/>
	  		<xsl:with-param name="text-y-axis" 		select="'Test Count per Source Statements'"/>
	  		<xsl:with-param name="width" 			select="$graph-width"/>
	  		<xsl:with-param name="height" 			select="$graph-height"/>
			<xsl:with-param name="log"				select="$log"/>
	  		<xsl:with-param name="point-nodeset">
	  			<points>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
		  				<!-- Calculating the y-coordinate -->
		  				<xsl:variable 	name="sum-of-tests"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/test-metrics/tests)"/>
		  				<xsl:variable 	name="sum-of-ncss"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss)"/>"
		  				<!-- in case of div by 0 -->
		  				<xsl:variable name="y-coordinate">
		  					<xsl:choose>
		  						<xsl:when test="$sum-of-ncss != 0">
		  							<xsl:value-of select="number($sum-of-tests) div number($sum-of-ncss)"/>
		  						</xsl:when>
		  						<xsl:otherwise>0</xsl:otherwise>
		  					</xsl:choose>
		  				</xsl:variable>
		  				<!-- logging everythin if we have to -->
		  				<xsl:choose>
		  					<xsl:when test="$log = 'true'">
		  						<xsl:message>
		  							Subsystem name:<xsl:value-of select="$subsystem-name"/>
		  							sum-of-ssc<xsl:value-of select="$sum-of-tests"/>
		  							sum-of-ncss<xsl:value-of select="$sum-of-ncss"/>
		  							product:<xsl:value-of select="$y-coordinate"/>
		  						</xsl:message>
		  					</xsl:when>
		  				</xsl:choose>
		  				<point>
		  					<x-label><xsl:value-of select="$subsystem-name"/></x-label>
		  					<y-coordinate><xsl:value-of select="$y-coordinate"/></y-coordinate>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="../.."/>
                                  </xsl:call-template></link>
		  				</point>
		  			</xsl:for-each>
	  			</points>
	  		</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="packages" mode="subsystem-test-coverage-histogram-graph">
		<xsl:variable name="packages-element" select="current()"/>

		<xsl:call-template name="simple-histogram-graph">
	  		<xsl:with-param name="text-x-axis" select="'Subsystem'"/>
	  		<xsl:with-param name="text-y-axis" select="'Test Coverage'"/>
	  		<xsl:with-param name="width" select="$graph-width"/>
	  		<xsl:with-param name="height" select="$graph-height"/>
			<xsl:with-param name="log"	  select="$log"/>
	  		<xsl:with-param name="point-nodeset">
	  			<points>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
		  				<!-- Calculating the y-coordinate -->
		  				<xsl:variable 	name="sum-of-ssc"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/test-metrics[not(source-statements-covered ='NaN')
		  								and
		  									not(source-statements-covered ='')]/source-statements-covered)"/>
		  				<xsl:variable 	name="sum-of-ncss"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss)"/>"
		  				<!-- in case of div by 0 -->
		  				<xsl:variable name="y-coordinate">
		  					<xsl:choose>
		  						<xsl:when test="$sum-of-ncss != 0">
		  							<xsl:value-of select="number($sum-of-ssc) div number($sum-of-ncss)"/>
		  						</xsl:when>
		  						<xsl:otherwise>0</xsl:otherwise>
		  					</xsl:choose>
		  				</xsl:variable>
		  				<!-- logging everythin if we have to -->
		  				<xsl:choose>
		  					<xsl:when test="$log = 'true'">
		  						<xsl:message>
		  							Subsystem name:<xsl:value-of select="$subsystem-name"/>
		  							sum-of-ssc<xsl:value-of select="$sum-of-ssc"/>
		  							sum-of-ncss<xsl:value-of select="$sum-of-ncss"/>
		  							product:<xsl:value-of select="$y-coordinate"/>
		  						</xsl:message>
		  					</xsl:when>
		  				</xsl:choose>


		  				<point>
		  					<x-label><xsl:value-of select="$subsystem-name"/></x-label>
		  					<y-coordinate><xsl:value-of select="$y-coordinate"/></y-coordinate>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></link>
		  				</point>
		  			</xsl:for-each>
	  			</points>
	  		</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="packages" mode="subsystem-size-histogram-graph">
		<xsl:variable name="packages-element" select="current()"/>

		<xsl:call-template name="simple-histogram-graph">
	  		<xsl:with-param name="text-x-axis" select="'Subsystem'"/>
	  		<xsl:with-param name="text-y-axis" select="'Source Statements'"/>
	  		<xsl:with-param name="width" select="$graph-width"/>
	  		<xsl:with-param name="height" select="$graph-height"/>
	  		<xsl:with-param name="point-nodeset">
	  			<points>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
		  				<point>
		  					<x-label><xsl:value-of select="$subsystem-name"/></x-label>
		  					<y-coordinate><xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss)"/></y-coordinate>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></link>
		  				</point>
		  			</xsl:for-each>
	  			</points>
	  		</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="packages" mode="subsystem-complexity-per-method-histogram-graph">
		<xsl:variable name="packages-element" select="current()"/>
		<xsl:call-template name="simple-histogram-graph">
	  		<xsl:with-param name="text-x-axis" select="'Subsystem'"/>
	  		<xsl:with-param name="text-y-axis" select="'Complexity per Method'"/>
	  		<xsl:with-param name="width" select="$graph-width"/>
	  		<xsl:with-param name="height" select="$graph-height"/>
			<xsl:with-param name="log" select="$log"/>

	  		<xsl:with-param name="point-nodeset">
	  			<points>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
						<!-- Calculating the y-coordinate -->
		  				<xsl:variable 	name="sum-of-ccn"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ccn)"/>
		  				<xsl:variable 	name="sum-of-functions"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/functions)"/>"
		  				<!-- in case of div by 0 -->
		  				<xsl:variable name="y-coordinate">
		  					<xsl:choose>
		  						<xsl:when test="$sum-of-functions != 0">
		  							<xsl:value-of select="number($sum-of-ccn) div number($sum-of-functions)"/>
		  						</xsl:when>
		  						<xsl:otherwise>0</xsl:otherwise>
		  					</xsl:choose>
		  				</xsl:variable>
		  				<!-- logging everythin if we have to -->
		  				<xsl:choose>
		  					<xsl:when test="$log = 'true'">
		  						<xsl:message>
		  							Subsystem name:<xsl:value-of select="$subsystem-name"/>
		  							sum-of-ccn<xsl:value-of select="$sum-of-ccn"/>
		  							sum-of-functions<xsl:value-of select="$sum-of-functions"/>
		  							product:<xsl:value-of select="$y-coordinate"/>
		  						</xsl:message>
		  					</xsl:when>
		  				</xsl:choose>

						<point>
		  					<x-label><xsl:value-of select="$subsystem-name"/></x-label>
		  					<y-coordinate><xsl:value-of select="$y-coordinate"/></y-coordinate>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></link>
		  				</point>
		  			</xsl:for-each>
	  			</points>
	  		</xsl:with-param>
		</xsl:call-template>
	</xsl:template>


	<xsl:template match="packages" mode="subsystem-violations-per-ncss-histogram-graph">
		<xsl:variable name="packages-element" select="current()"/>
		<xsl:call-template name="simple-histogram-graph">
	  		<xsl:with-param name="text-x-axis" select="'Subsystem'"/>
	  		<xsl:with-param name="text-y-axis" select="'Violations per Source Statements'"/>
	  		<xsl:with-param name="width" select="$graph-width"/>
	  		<xsl:with-param name="height" select="$graph-height"/>
			<xsl:with-param name="log" select="$log"/>
	  		<xsl:with-param name="point-nodeset">
	  			<points>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
		  				<!-- Calculating the y-coordinate -->
		  				<xsl:variable 	name="sum-of-violations"
					  					select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/code-violations)"/>
		  				<xsl:variable 	name="sum-of-ncss"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss)"/>"
		  				<!-- in case of div by 0 -->
		  				<xsl:variable name="y-coordinate">
		  					<xsl:choose>
		  						<xsl:when test="$sum-of-violations != 0">
		  							<xsl:value-of select="number($sum-of-violations) div number($sum-of-ncss)"/>
		  						</xsl:when>
		  						<xsl:otherwise>0</xsl:otherwise>
		  					</xsl:choose>
		  				</xsl:variable>
		  				<!-- logging everythin if we have to -->
		  				<xsl:choose>
		  					<xsl:when test="$log = 'true'">
		  						<xsl:message>
		  							Subsystem name:<xsl:value-of select="$subsystem-name"/>
		  							sum-of-violations<xsl:value-of select="$sum-of-violations"/>
		  							sum-of-ncss<xsl:value-of select="$sum-of-ncss"/>
		  							product:<xsl:value-of select="$y-coordinate"/>
		  						</xsl:message>
		  					</xsl:when>
		  				</xsl:choose>
		  				<point>
		  					<x-label><xsl:value-of select="$subsystem-name"/></x-label>
		  					<y-coordinate><xsl:value-of select="$y-coordinate"/></y-coordinate>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></link>
		  				</point>
		  			</xsl:for-each>
	  			</points>
	  		</xsl:with-param>
		</xsl:call-template>

	</xsl:template>

    <xsl:template match="packages" mode="subsystem-source-control-histogram-graph">
        <xsl:param name="sc-metric"/>
        <xsl:param name="y-axis-name"/>
        <xsl:variable name="packages-element" select="current()"/>
        <xsl:call-template name="simple-histogram-graph">
            <xsl:with-param name="text-x-axis" select="'Subsystem'"/>
            <xsl:with-param name="text-y-axis" select="$y-axis-name"/>
            <xsl:with-param name="width" select="$graph-width"/>
            <xsl:with-param name="height" select="$graph-height"/>
            <xsl:with-param name="point-nodeset">
                <points>
                    <xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
                        <xsl:variable name="subsystem-name" select="normalize-space(name)"/>
                        <xsl:variable name="error-activity" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/error-activity)"/>
                        <xsl:variable name="change-activity" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/change-activity)"/>
                        <xsl:variable name="y-coordinate">
                            <xsl:choose>
                                <xsl:when test="$sc-metric='total-activity'">
                                    <xsl:value-of select="$error-activity + $change-activity"/>
                                </xsl:when>
                                <xsl:when test="$sc-metric='error-percentage'">
                                    <xsl:choose>
                                        <xsl:when test="$error-activity=0">0</xsl:when>
                                        <xsl:otherwise><xsl:value-of select="100 * $error-activity div ($error-activity + $change-activity)"/></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <!-- for other metrics, just sum the elements with the specified name -->
                                    <xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/node()[name() = $sc-metric])"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <point>
                            <x-label><xsl:value-of select="$subsystem-name"/></x-label>
                            <y-coordinate><xsl:value-of select="$y-coordinate"/></y-coordinate>
                            <link>
                                <xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                </xsl:call-template>
                            </link>
                        </point>
                    </xsl:for-each>
                </points>
            </xsl:with-param>
        </xsl:call-template>
    </xsl:template>


	<xsl:template match="packages" mode="subsystem-style-errors-per-ncss-histogram-graph">
		<xsl:variable name="packages-element" select="current()"/>

		<xsl:call-template name="simple-histogram-graph">
	  		<xsl:with-param name="text-x-axis" 	select="'Subsystem'"/>
	  		<xsl:with-param name="text-y-axis" 	select="'Style Errors per Source Statements'"/>
	  		<xsl:with-param name="width" 		select="$graph-width"/>
	  		<xsl:with-param name="height" 		select="$graph-height"/>
			<xsl:with-param name="log"			select="$log"/>
	  		<xsl:with-param name="point-nodeset">
	  			<points>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  				<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
		  				<!-- Calculating the y-coordinate -->
		  				<xsl:variable 	name="sum-of-violations"
		  								select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/style-errors)"/>
		  				<xsl:variable 	name="sum-of-ncss"
					  					select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss)"/>"
		  				<!-- in case of div by 0 -->
		  				<xsl:variable name="y-coordinate">
		  					<xsl:choose>
		  						<xsl:when test="$sum-of-violations != 0">
		  							<xsl:value-of select="number($sum-of-violations) div number($sum-of-ncss)"/>
		  						</xsl:when>
		  						<xsl:otherwise>0</xsl:otherwise>
		  					</xsl:choose>
		  				</xsl:variable>
		  				<!-- logging everythin if we have to -->
		  				<xsl:choose>
		  					<xsl:when test="$log = 'true'">
		  						<xsl:message>
		  							Subsystem name:<xsl:value-of select="$subsystem-name"/>
		  							sum-of-violations<xsl:value-of select="$sum-of-violations"/>
		  							sum-of-ncss<xsl:value-of select="$sum-of-ncss"/>
		  							product:<xsl:value-of select="$y-coordinate"/>
		  						</xsl:message>
		  					</xsl:when>
		  				</xsl:choose>
		  				<point>
		  					<x-label><xsl:value-of select="$subsystem-name"/></x-label>
		  					<y-coordinate><xsl:value-of select="$y-coordinate"/></y-coordinate>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></link>
		  				</point>
		  			</xsl:for-each>
	  			</points>
	  		</xsl:with-param>
		</xsl:call-template>

	</xsl:template>

	<xsl:template match="packages" mode="subsystem-dependency-graph">
		<xsl:variable name="packages-element" select="current()"/>

        <xsl:variable name="total-packages" select="count($packages-element/package)"/>
        <xsl:variable name="total-classes" select="sum($packages-element/package/statistics/source-metrics/total-classes)"/>
        <xsl:variable name="total-ncss" select="sum($packages-element/package/statistics/source-metrics/ncss)"/>
        <xsl:variable name="total-ccn" select="sum($packages-element/package/statistics/source-metrics/ccn)"/>

		<xsl:call-template name="dependency-graph">
	  		<xsl:with-param name="width" select="800"/>
	  		<xsl:with-param name="height" select="400"/>
	  		<xsl:with-param name="subsystems-nodeset">
	  			<subsystems>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  				    <xsl:variable name="subsystem-name" select="normalize-space(name)"/>
		  				    <xsl:variable name="subsystem-total-packages" select="count($packages-element/package[@subsystem = $subsystem-name])"/>
                            <xsl:variable name="subsystem-total-classes" select="sum($packages-element/package[@subsystem = $subsystem-name]/statistics/source-metrics/total-classes)"/>
                            <xsl:variable name="subsystem-total-ncss" select="sum($packages-element/package[@subsystem = $subsystem-name]/statistics/source-metrics/ncss)"/>
                            <xsl:variable name="subsystem-total-ccn" select="sum($packages-element/package[@subsystem = $subsystem-name]/statistics/source-metrics/ccn)"/>
                            <xsl:variable name="subsystem-total-ncss-covered" select="sum($packages-element/package[@subsystem = $subsystem-name]/statistics/test-metrics[not(source-statements-covered ='NaN') and not(source-statements-covered ='')]/source-statements-covered)"/>
                          <xsl:copy>
		  					<name><xsl:value-of select="$subsystem-name"/></name>
		  					<description><xsl:value-of select="normalize-space(description)"/></description>
                            <level><xsl:value-of select="@level"/></level>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                    <xsl:with-param name="path-to-root" select="$path-to-root"/>
                                  </xsl:call-template></link>
		  					<illegal-dependencies>
		  						<xsl:value-of select="sum($packages-element/package[@subsystem = $subsystem-name]/statistics/dependancy-metrics/subsystem[@legal !='true'])"/>
		  					</illegal-dependencies>
		  					<legal-dependencies>
		  						<xsl:value-of select="sum($packages-element/package[@subsystem = $subsystem-name]/statistics/dependancy-metrics/subsystem[@legal ='true'])"/>
		  					</legal-dependencies>
                            <statistics>
                                <stat   name="Packages"
                                        value="{concat($subsystem-total-packages,' (',format-number($subsystem-total-packages div $total-packages, '##.#%'),')')}"/>
                                <stat   name="Classes"
                                        value="{concat($subsystem-total-classes,' (',format-number($subsystem-total-classes div $total-classes, '##.#%'),')')}"/>
                                <stat   name="Source Statements"
                                        value="{concat($subsystem-total-ncss,' (',format-number($subsystem-total-ncss div $total-ncss, '##.#%'),')')}"/>
                                <stat   name="Cyclomatic Complexity"
                                        value="{concat($subsystem-total-ccn,' (',format-number($subsystem-total-ccn div $total-ccn, '##.#%'),')')}"/>
                                <xsl:if test="$subsystem-total-ncss-covered &gt; 0">
                                    <stat   name="Unit Test Coverage"
                                            value="{format-number($subsystem-total-ncss-covered div $subsystem-total-ncss, '##.#%')}"/>
                                </xsl:if>
                                <!--stat name="Unit Test Coverage" value="1"/-->
                            </statistics>

		  					<subsystem-dependences>
		  					<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		  						<xsl:variable name="this-subsystem-id" select="@id"/>
		  						<xsl:variable name="this-subsystem-name" select="normalize-space(name)"/>
		  						<xsl:variable name="dependencies-to-this-subsystem" select="sum($packages-element/package[@subsystem=$subsystem-name]/statistics/dependancy-metrics/subsystem[@name = $this-subsystem-name])"/>
		  						<subsystem position="{position()}" level="{@level}">
		  							<xsl:choose>
			  							<xsl:when test="not(count($radar-config-doc/radar-config/subsystems/subsystem[normalize-space(name) = $subsystem-name]/legal-subordinates/subsystem[@id = $this-subsystem-id]) &gt; 0)">
											<xsl:attribute name="legal">false</xsl:attribute>
										</xsl:when>
										<xsl:otherwise>
											<xsl:attribute name="legal">true</xsl:attribute>
										</xsl:otherwise>
									</xsl:choose>
									<xsl:value-of select="$dependencies-to-this-subsystem"/>
		  						</subsystem>
		  					</xsl:for-each>
		  					</subsystem-dependences>

		  				</xsl:copy>
		  			</xsl:for-each>
	  			</subsystems>
	  		</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

    <xsl:template match="packages" mode="subsystem-quality-graph">
    	<xsl:param name="log"/>
		<xsl:variable name="packages-element" select="current()"/>

		<xsl:call-template name="subsystem-graph">
	  		<xsl:with-param name="width" select="800"/>
	  		<xsl:with-param name="height" select="800"/>
	  		<xsl:with-param name="subsystems-nodeset">
	  			<subsystems>
		  			<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
                          <xsl:variable name="subsystem-name" select="name"/>
						  <xsl:choose>
								<xsl:when test="$log = 'true'">
		                          <xsl:message>
		                         	subsys:<xsl:value-of select="concat($docs-home,'/xml/reports/subsystem_quality_report_',normalize-space($subsystem-name),'.xml')"/>
		                          </xsl:message>
		                         </xsl:when>
		                  </xsl:choose>
                          <xsl:variable name="system-quality-report" select="document(concat('file://',concat($docs-home,'/xml/reports/subsystem_quality_report_',normalize-space($subsystem-name),'.xml')))"/>
                          <xsl:copy>
		  					<name><xsl:value-of select="$subsystem-name"/></name>
		  					<description><xsl:value-of select="normalize-space(description)"/></description>
                            <level><xsl:value-of select="@level"/></level>
		  					<link><xsl:call-template name="svg-subsystem-url">
                                <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                                <xsl:with-param name="path-to-root" select="$path-to-root"/>
                              </xsl:call-template></link>
		  					<color><xsl:value-of select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@max-value &gt; $system-quality-report/metric/value]/@color"/></color>
		  				</xsl:copy>
		  			</xsl:for-each>
	  			</subsystems>
	  		</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="packages" mode="abstractness-instability-graph">
		<xsl:variable name="packages-element" select="current()"/>

		<xsl:call-template name="abstractness-instability-graph">
	  		<xsl:with-param name="text-x-axis" select="'Instability'"/>
	  		<xsl:with-param name="text-y-axis" select="'Abstractness'"/>
	  		<xsl:with-param name="width" select="$graph-height"/>
	  		<xsl:with-param name="height" select="$graph-height"/>
	  		<xsl:variable   name="color-ok"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 3]/@color"/>
			<xsl:variable   name="color-warn"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 2]/@color"/>
			<xsl:variable   name="color-ko"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 1]/@color"/>

			<xsl:with-param name="point-nodeset">
	  			<points>
		  			<xsl:for-each select="package[statistics/package-oo-metrics/instability]">
		  				<point>
		  					<label><xsl:value-of select="@name"/></label>
                            <link><xsl:call-template name="package-url">
                                <xsl:with-param name="package-name" select="@name"/>
                                <xsl:with-param name="path-to-root" select="$path-to-root"/>
                            </xsl:call-template></link>
		  					<distance><xsl:value-of select="statistics/package-oo-metrics/distance"/></distance>
		  					<instability><xsl:value-of select="statistics/package-oo-metrics/instability"/></instability>
		  					<abstractness><xsl:value-of select="statistics/package-oo-metrics/abstractness"/></abstractness>
		  				</point>
		  			</xsl:for-each>
	  			</points>
	  		</xsl:with-param>
	  		<xsl:with-param name="max-x" select="'1'"/>
		  	<xsl:with-param name="max-y" select="'1'"/>
		</xsl:call-template>
	</xsl:template>

	<!-- TODO: Finish this to factorize some part of the above code
				this may have done somewhere else... maybe in the math
				stylesheets ?

	<xsl:template name="division">
		Calculating the y-coordinate
		<xsl:param  name="label"/>
		<xsl:param 	name="left-operand"/>
		<xsl:param 	name="right-operand"/>
		<xsl:param 	name="path-to-root"/>
		<xsl:param  name="log"/>
		in case of div by 0
		<xsl:variable name="product">
			<xsl:choose>
				<xsl:when test="$right-operand != 0">
					<xsl:value-of select="number($left-operand) div number($right-operand)"/>
				</xsl:when>
				<xsl:otherwise>0</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		logging everythin if we have to
		<xsl:choose>
			<xsl:when test="$log = 'true'">
				<xsl:message>
					Subsystem name:<xsl:value-of select="$label"/>
					left-operand<xsl:value-of select="$left-operand"/>
					right-operand<xsl:value-of select="$right-operand"/>
					product:<xsl:value-of select="$produt"/>
				</xsl:message>
			</xsl:when>
		</xsl:choose>
		generating the xml required
		<point>
			<x-label><xsl:value-of select="$label"/></x-label>
			<y-coordinate><xsl:value-of select="number($left-operand) div number($right-operand)"/></y-coordinate>
			<link><xsl:call-template name="svg-subsystem-url">
				<xsl:with-param name="subsystem-name" select="$label"/>
				<xsl:with-param name="path-to-root" select="$path-to-root"/>
			</xsl:call-template></link>
		</point>

	</xsl:template> -->

</xsl:stylesheet>
